# -*- coding: utf-8 -*-
"""
God class about wrapping/hacking/enhancing subscription-manager/cp-subscription-manager
"""
import sys
import os
import re
import datetime
import zipfile
import logging
import logging.config
import errno
import subprocess
import configparser
import traceback
import datetime
import dateutil.parser
import platform


LOGFILE_PATH = u"/var/log/vzsubscribe/vzsubscribe.log"
CACHE_DB = u"/var/cache/vzsubscribe/vzsubscribe.ini"
CONTRACTS_DB = u"/var/cache/vzsubscribe/contracts.ini"
EULA_ACCEPTED = u"/var/cache/vzsubscribe/eula_accepted"
FAILS_TO_ATTACH_POOL = 7

EULA_FILE = "/usr/share/licenses/vzsubscribe/EULA"

#pylint: disable=W0107, C0103

class More(object):
    '''
      Class to print long text with "more"-like functionality
    '''
    def __init__(self, num_lines):
        self.num_lines = num_lines-1

    def __ror__(self, other):
        '''
          Pipe operator
        '''  
        s = str(other).split("\n")
        for i in xrange(0, len(s), self.num_lines):
            print "\n".join(s[i:i+self.num_lines])
            raw_input("Press <Enter> for more")


ZERO = datetime.timedelta(0)

class UTC(datetime. tzinfo):
  '''
  Small ugly snippet to fix UTC-issue with python datetime conversion.
  '''
  def utcoffset(self, dt):
    return ZERO

  def tzname(self, dt):
    
    return "UTC"

  def dst(self, dt):
    return ZERO

UTC_TZ = UTC()

class ZipAll(object):
    '''
    Lazy way to add files and folders to zip
    '''
    def __init__(self, zipname):
        '''
        Open Zipfilename
        '''
        self.zf = zipfile.ZipFile(zipname, 'w')
        pass
        
    def to_zip(self, filename2store):
        '''
        Add file or folder to zipfile
        '''
        if not os.path.exists(filename2store):
            return

        if os.path.isfile(filename2store):
            self.zf.write(filename2store)

        if os.path.isdir(filename2store):
            #pylint: disable=W0612
            for root, _dirnames, filenames in os.walk(filename2store):
                for filename in filenames:
                    filepath = os.path.realpath(os.path.join(root, filename))
                    self.zf.write(filepath)
                    
                    
    def close(self):
        '''
        Close ZIP file.
        '''
        self.zf.close()


def logmethod(method):
    """
    Decorator to log call end exists from methods
    """
    def _wrapper(self, *args, **kwds):
        "Use this one for instance or class methods"
        self.log.debug("BEGIN " + method.__name__)
        ret = method(self, *args, **kwds)
        self.log.debug("END " + method.__name__)
        return ret
    return _wrapper

def file2string(filepath):
    """
    Read file into string
    """
    lstr = None
    if os.path.exists(filepath):
        lfile = open(filepath, "r")
        lstr = lfile.read()
        lfile.close()
    return lstr


def str2file(str_, filename):
    '''
    Short writing given string to the file.
    '''
    lfile = open(filename, 'w')
    lfile.write(str_)
    lfile.close()

def mkdir_p(path):
    """
    Create path if not exists
    """
    try:
        os.makedirs(path)
    except OSError as exc: # Python >2.5
        if exc.errno == errno.EEXIST and os.path.isdir(path):
            pass
        else:
            raise
    pass            

class VZSM(object):
    '''
    Business logic 
    '''
    def __init__(self):
        '''
        Getting base info about current SM subscription
        '''
        self.last_return_code = 0
        linux_distribution = platform.linux_distribution()
        self.use_cpsm = False

        if 'Red Hat' in linux_distribution[0]:
            self.use_cpsm = True

        if os.path.exists('/usr/bin/cp-subscription-manager') and not os.path.exists('/usr/bin/subscription-manager'):
            self.use_cpsm = True

        self.sm_name = "subscription-manager"
        self.repo_filepath = '/etc/yum.repos.d/redhat.repo'

        if self.use_cpsm:
            import cpsm.config
            self.cfg = cpsm.config.initConfig()
            self.sm_name = "cp-subscription-manager"
            self.repo_filepath = '/etc/yum.repos.d/cpsm.repo'
            sys.path.append('/usr/share/cpsm')
            import cp_subscription_manager.injection as inj
            from cp_subscription_manager.certdirectory import EntitlementDirectory
            inj.provide(inj.ENT_DIR, EntitlementDirectory, singleton=True)
            self.inj = inj
        else:
            import rhsm.config
            self.cfg = rhsm.config.initConfig()
            sys.path.append('/usr/share/rhsm')
            import subscription_manager.injection as inj
            from subscription_manager.certdirectory import EntitlementDirectory
            inj.provide(inj.ENT_DIR, EntitlementDirectory, singleton=True)
            self.inj = inj

        mkdir_p(os.path.dirname(LOGFILE_PATH))
        mkdir_p(os.path.dirname(CACHE_DB))

        self.cache_ini = configparser.ConfigParser()
        try:    
            self.cache_ini.read(CACHE_DB)
        except Exception:
            pass

        if not u'contracts' in self.cache_ini:
            self.cache_ini[u'contracts'] = {}

        if not u'prolongate_frequency_minutes' in self.cache_ini[u'contracts']:
            self.cache_ini[u'contracts'][u'prolongate_frequency_minutes'] = u'240'

        #     logging.basicConfig(filename=LOGFILE_PATH,
        #                         level=logging.DEBUG,
        #                         format='%(asctime)s:%(levelname)s:%(module)s> %(message)s')
        self.log = logging.getLogger("VZSM")
        self.log.setLevel(logging.DEBUG)
        handler = logging.handlers.RotatingFileHandler(LOGFILE_PATH, maxBytes=20*1024, backupCount=10)
        formatter = logging.Formatter('%(asctime)s:%(levelname)s:%(module)s> %(message)s')
        handler.setFormatter(formatter)
        self.log.addHandler(handler)
        # self.reload()
        self.save_cache()

        yum_vars_dir = "/etc/yum/vars"
        mkdir_p(yum_vars_dir)
        
        osname_varfile = os.path.join(yum_vars_dir, "cpsm_osname")
        osversion_varfile = os.path.join(yum_vars_dir, "cpsm_osversion")

        #normalization
        dist_name = linux_distribution[0].lower()
        self.dist_name = '-'.join(dist_name.split())

        if not os.path.exists(osname_varfile):
            with open(osname_varfile, "w") as lf_:
              lf_.write(self.dist_name)
      
        if not os.path.exists(osversion_varfile):
            dist_major_version = linux_distribution[1].split('.')[0]
            with open(osversion_varfile, "w") as lf_:
              lf_.write(dist_major_version)

        pass        

    @logmethod
    def save_cache(self):
        '''
        Save some info in CACHE_DB
        '''
        try:
            with open(CACHE_DB, 'w') as file_:
                self.cache_ini.write(file_)
        except Exception as ex_:
            error_msg = '\n'.join([str(ex_), traceback.format_exc()])
            self.log.error(error_msg)
            return False
        return True


    @logmethod
    def sync_contracts_db(self):
        '''
        Sync contracts info using calls to subscription_manager libs
        No remote call to candlepin should be made.
        '''
        try:
            contracts_ini = configparser.ConfigParser()
            try:    
                contracts_ini.read(CONTRACTS_DB)
            except Exception:
                pass
            for contract in contracts_ini.sections():
                contracts_ini.set(contract, u'ok', u'')
                try:
                    fail_to_attach = int(contracts_ini.get(contract, u'fail_to_attach'))
                    if fail_to_attach > FAILS_TO_ATTACH_POOL:
                        contracts_ini.remove_section(contract)
                except Exception:
                    pass

            entitlement_dir = self.inj.require(self.inj.ENT_DIR)
            # pooltype_cache = inj.require(inj.POOLTYPE_CACHE)
            certs = entitlement_dir.list()
            if len(certs):
                for cert in certs:
                    order = cert.order
                    if order:
                        contract = order.contract #or ""
                        if  contract not in contracts_ini:
                            contracts_ini[contract] = {}

                        contracts_ini.set(contract, u'quantity', unicode(order.quantity_used))
                        pool_id = ''
                        if hasattr(cert.pool, "id"):
                            pool_id = cert.pool.id
                        contracts_ini.set(contract, u'pool_id', pool_id)
                        contracts_ini.set(contract, u'end_date', unicode(cert.valid_range.end().isoformat()))
                        contracts_ini.remove_option(contract, u'fail_to_attach')
                        contracts_ini.set(contract, u'ok', u'1')
            with open(CONTRACTS_DB, 'w') as configfile:    # save
                contracts_ini.write(configfile)
            pass
        except Exception as ex_:
            error_msg = '\n'.join([str(ex_), traceback.format_exc()])
            self.log.error(error_msg)
            return False
        return True

    @logmethod
    def try_to_prolongate_contracts(self, force=False):
        '''
        For current contracts, that expired or out ouf pool_id
        and attach all available pools with same contract numbers
        '''
        try:
            need_sync = False
            if force:
                need_sync = True
            else:
                if u'last_sync_datetime' not in self.cache_ini['contracts']:
                    need_sync  = True
                else:
                    lsd = self.cache_ini['contracts']['last_sync_datetime']
                    try:
                        last_sync_datetime  = dateutil.parser.parse(lsd)
                        prolongate_frequency_minutes = int(self.cache_ini['contracts']['prolongate_frequency_minutes'])
                        now = datetime.datetime.now(UTC_TZ)
                        if now > last_sync_datetime + datetime.timedelta(hours=prolongate_frequency_minutes) or now < last_sync_datetime:
                            need_sync  = True
                    except:
                        need_sync  = True

            if not need_sync:
                return True

            contracts_ini = configparser.ConfigParser()
            try:    
                contracts_ini.read(CONTRACTS_DB)
            except Exception:
                pass

            contracts_need_pool = []
            for contract in contracts_ini.sections():
                contract_need_pool = False
                ok = contracts_ini.get(contract, 'ok')
                if not ok == '1':
                    contract_need_pool = True
                end_date = dateutil.parser.parse(contracts_ini.get(contract, 'end_date'))
                if end_date + datetime.timedelta(hours=12) < datetime.datetime.now(UTC_TZ):
                    contract_need_pool = True

                if contract_need_pool:
                    contracts_need_pool.append(contract)

            if contracts_need_pool:
                available_products_text = self.get_available_products_text()
                for contract_id in contracts_need_pool:
                    restr = r"(?m)Contract:\s+%s\s*\nPool ID:\s+(?P<poolid>[0-f]+)" % contract_id
                    re_ = re.compile(restr)
                    for poolid in re_.findall(available_products_text):
                        scmd = '%s attach --pool=%s ' % (self.sm_name, poolid)
                        #pylint: disable=W0612
                        res = self.get_output(scmd)
                        contracts_ini.set(contract_id, u'fail_to_attach', u'0')
                    else:
                        fail_to_attach = 0
                        try:
                            fail_to_attach = int(contracts_ini.get(contract_id, u'fail_to_attach'))
                        except Exception as ex_:
                            pass
                        fail_to_attach += 1
                        contracts_ini.set(contract_id, u'fail_to_attach', unicode(fail_to_attach))
                        with open(CONTRACTS_DB, 'w') as configfile:    # save
                            contracts_ini.write(configfile)
                        pass
            
            self.cache_ini[u'contracts'][u'last_sync_datetime'] = unicode(datetime.datetime.now(UTC_TZ).isoformat())
            self.save_cache()

        except Exception as ex_:
            error_msg = '\n'.join([str(ex_), traceback.format_exc()])
            self.log.error(error_msg)
            return False
        return True



    @logmethod
    def get_output(self, scmd):
        '''
        Just get output of OS command.
        Log commands and results
        '''
        #pylint: disable=W0612
        self.log.debug(scmd)
        # progin, progout = os.popen4(scmd)
        
        proc = subprocess.Popen(scmd, shell=True,
                stdin=subprocess.PIPE,
                stdout=subprocess.PIPE,
                stderr=subprocess.STDOUT,
                close_fds=True)
        out, _ = proc.communicate()
        proc.wait()
        self.last_return_code = proc.returncode
        self.log.debug(out)
        return out

    # @logmethod
    # def get_org_id(self):
    #     '''
    #     Get ID of currently attached organization if system registered
    #     '''
    #     org_id = None
    #     re_ = re.compile(r"(?m)org ID:\s+(?P<orgid>.+?)\s*$")
    #     for org_id in re_.findall(self.get_identity_text()):
    #         break
    #     return org_id


    @logmethod
    def get_current_products_text(self):
        '''
            [root@localhost yum.repos.d]# subscription-manager list
    +-------------------------------------------+
        Installed Product Status
    +-------------------------------------------+
    Product Name:    Content Product for /path/to/content14
    Product ID:     201608723337
    Version:        
    Arch:           
    Status:         Subscribed
    Status Details: 
    Starts:         
    Ends:
        '''
        list_text = self.get_output('%s list --consumed' % self.sm_name)
        return list_text 

    @logmethod
    def get_consumed_products_text(self):
        '''
    +
    [root@localhost yum.repos.d]# subscription-manager list --consumed
    +-------------------------------------------+
       Consumed Subscriptions
    +-------------------------------------------+
    Subscription Name:   MKT201608723339
    Provides:            Content Product for /path/to/content16
    SKU:                 MKT201608723339
    Contract:            
    Account:             
    Serial:              2241009699337863849
    Pool ID:             00000000528f5ee90152a01a787812b4
    Provides Management: No
    Active:              True
    Quantity Used:       1
    Service Level:       
    Service Type:        
    Status Details:      Subscription is current
    Subscription Type:   
    Starts:              02/02/2016
    Ends:                02/02/2017
    System Type:         Physical
        '''
        list_text = self.get_output('%s list --consumed' % self.sm_name)
        return list_text

    @logmethod
    def get_identity_text(self):
        '''
        Identity text
        '''
        list_text = self.get_output('%s identity' % self.sm_name)
        return list_text 

    
    @logmethod
    def get_available_products_text(self):
        '''
        Asks SM about available subsription pools
        '''
        scmd = '%s list --available' % self.sm_name
        text_available = self.get_output(scmd)
        return text_available
    
    # @logmethod
    # def get_svc_products_ids(self):
    #     '''
    #     Collect ids of currently consumed engeeniring products (SVC)
    #     '''
    #     product_ids = set()
    #     re_ = re.compile(r"(?m)Product ID:\s+(?P<id>\d+)\s*$")
    #     for id_ in re_.findall(self.get_current_products_text()):
    #         product_ids.add(id_)
    # 
    #     re_ = re.compile(r"(?m)Subscription Name:\s+MKT(?P<id>\d+)\s*$")
    #     for id_ in re_.findall(self.get_consumed_products_text()):
    #         product_ids.add(id_)
    #         
    #     return product_ids

    @logmethod
    def get_contracts_numbers(self):
        '''
        Collect contract numbers from consumed subscriptions
        '''
        contract_ids = set()
        re_ = re.compile(r"(?m)Contract:\s+(?P<id>\d+)\s*$")
        for id_ in re_.findall(self.get_consumed_products_text()):
            contract_ids.add(id_)

        return contract_ids

        
    # @logmethod
    # def reload(self):
    #     '''
    #     Reload subscription stuff
    #     '''
    #     try:
    #         self.owner_id = self.get_org_id()
    #         self.product_ids = self.get_svc_products_ids()
    #     except Exception as ex_:
    #         error_msg = '\n'.join([str(ex_), traceback.format_exc()])
    #         self.log.error(error_msg)
    #         return False
    #     return True
        

    # @logmethod
    # def restore_product_certificates(self):
    #     '''
    #     If host subscribed we get list of consumed products
    #     and get product certificates in 
    #     '''
    #     if not self.owner_id or not self.product_ids:
    #         return
    #     
    #     try:
    #         cfg = self.cfg
    #         owner_id = self.owner_id #pylint: disable=W0612
    #         consumer_cert_dir = cfg.get('rhsm', 'consumerCertDir')
    #         conn = connection.UEPConnection(
    #             host=cfg.get('server', 'hostname'),
    #             ssl_port=cfg.get_int('server', 'port'),
    #             handler=cfg.get('server', 'prefix'),
    #             proxy_hostname=cfg.get('server', 'proxy_hostname'),
    #             proxy_port=cfg.get_int('server', 'proxy_port'),
    #             proxy_user=cfg.get('server', 'proxy_user'),
    #             proxy_password=cfg.get('server', 'proxy_password'),
    #             cert_file=os.path.join(consumer_cert_dir, 'cert.pem'),
    #             key_file=os.path.join(consumer_cert_dir, 'key.pem'))
    #     
    #         product_cert_dir = cfg.get('rhsm', 'productCertDir')
    #         for svc_id in self.product_ids:
    #             prefix_ = os.path.join(product_cert_dir, svc_id)
    #             cert_filename = prefix_  + '.pem'
    #             key_filename = prefix_  + '-key.pem'
    #             if not (os.path.exists(cert_filename) and
    #                     os.path.exists(key_filename)):
    #                 cmd = '/owners/%(owner_id)s/products/%(svc_id)s/certificate' % vars()
    #                 self.log.debug(cmd)
    #                 res = conn.conn.request_get(cmd)
    #                 self.log.debug(res)
    #                 if 'cert' in res and 'key' in res:
    #                     cert = res['cert']
    #                     key = res['key']
    #                     self.log.info("Getting certs for SVC Product " + svc_id)
    #                     prefix_ = os.path.join(consumer_cert_dir, svc_id)
    #                     str2file(cert, cert_filename)
    #                     str2file(key, key_filename)
    #     except Exception as ex_:
    #         error_msg = '\n'.join([str(ex_), traceback.format_exc()])
    #         self.log.error(error_msg)
    #         return False
    #     return True

    # @logmethod
    # def autoattach_pools_by_contracts(self):
    #     '''
    #     We collect all contract numbers of currently consumed subscriptions
    #     and attach all available pools with same contract numbers
    #     '''
    #     try:
    #         contract_ids = self.get_contracts_numbers()
    #         for contract_id in contract_ids:
    #             restr = r"(?m)Contract:\s+%s\s*\nPool ID:\s+(?P<poolid>[0-f]+)" % contract_id
    #             re_ = re.compile(restr)
    #             for poolid in re_.findall(self.get_available_products_text()):
    #                 scmd = '%s attach --pool=%s ' % (self.sm_name, poolid)
    #                 #pylint: disable=W0612
    #                 res = self.get_output(scmd)
    #                 pass
    #     except Exception as ex_:
    #         error_msg = '\n'.join([str(ex_), traceback.format_exc()])
    #         self.log.error(error_msg)
    #         return False
    #     return True
    
    @logmethod
    def patch_sm_plugin(self):
        try:
            is_ok = self.get_repo_urls()
            smpluginconf = "/etc/yum/pluginconf.d/subscription-manager.conf"
            stext = open(smpluginconf, "r").read()
            if is_ok:
                stext = re.sub(r'enabled\s*=\s*0', 'enabled=1', stext)
            else:
                stext = re.sub(r'enabled\s*=\s*1', 'enabled=0', stext)
            open(smpluginconf, "wt").write(stext)
        except Exception as ex_:
            self.log.error("Failed to patch %s", smpluginconf)
            error_msg = '\n'.join([str(ex_), traceback.format_exc()])
            self.log.error(error_msg)
            return False
        return True
            
    @logmethod
    def heal(self, force=False):
        '''
        Trying to fix problems of prolongation/shortening of any subscription
        when we have to subscribe to new pool with right date.
        
        New pool for the subscription can be attached by two ways (for reliability):
        * If we store product certificates in /etc/pki/products,
          rhsmcertd try to "heal" and attach appropriate subscription
        * We directly trying to get any new pool with contract number same as in our subscriptions.
        '''
        # if not self.reload():
        #     return False
        
        if not self.sync_contracts_db():
            return False
        
        if not self.try_to_prolongate_contracts(force=force):
            return False

        # if not self.autoattach_pools_by_contracts():
        #     return False
            
        # if not self.restore_product_certificates():
        #     return False
            
        if not self.patch_sm_plugin():
            return False

        return True

    @logmethod
    def info(self, verbose=False):
        """
        Lot of info about state of subscription
        """
        def out_(msg):
            '''
            Print and log
            '''
            self.log.info(msg)
            print msg
            
        out_("=== Summary ===")
        out_(self.get_identity_text())
        self.repo_info()
        out_("=== Consumed Products ===")
        out_(self.get_consumed_products_text())
        
        if verbose:
            out_("=== Available Products ===")
            out_(self.get_available_products_text())
        return True
    
    @logmethod
    def diagnostics(self):
        """
        Prepare diagnostics pack
        """
        #pylint: disable=E1101
        now = datetime.datetime.now(UTC_TZ).strftime('%Y-%m-%d-%H-%M-%S')
        diagnostics_filename = 'vzsubscribe-diagnostics-pack-' + now + '.zip'
        
        za = ZipAll(diagnostics_filename)
        za.to_zip(self.cfg.config_file)
        
        for opt in ['productCertDir',
                    'entitlementCertDir',
                    'consumerCertDir',
                    'ca_cert_dir']:
            try:
                za.to_zip(self.cfg.get('rhsm', opt))
            except:
                pass
            
        for path in [LOGFILE_PATH,
                     '/var/log/rhsm/',
                     CACHE_DB,
                     CONTRACTS_DB,
                     self.cfg.config_file]:
            try:
                za.to_zip(path)
            except:
                pass
        za.close()
        
        txtinfo = '''
    Prepared diagnostics pack
       %(diagnostics_filename)s.
    If you have any problems with subscriptions,
    please, send it to our support.
        ''' % vars()
        self.log.info(txtinfo)
        print txtinfo 
        return True

    @logmethod
    def get_repo_urls(self):
        """
        Collect RPM repositories from redhat repo.
        """
        repo_urls = set()
        redhat_repo = file2string(self.repo_filepath)
        self.log.debug(redhat_repo)
        re_ = re.compile(r'(?m)baseurl\s*=\s*(?P<url>.+)\s*$')
        repo_urls = set()
        if redhat_repo:
            for url in re_.findall(redhat_repo):
                repo_urls.add(url)
        return repo_urls

    @logmethod
    def repo_info(self):
        """
        Print info about subscriptions
        """
        repo_urls = self.get_repo_urls()
        if repo_urls:
            info = ''.join(['The system successfully registered, attached repositories:\n  *  ',
                            "\n  *  ".join(repo_urls)])
            self.log.info(info)
            print info
            return True
        return False

    @logmethod
    def register(self, owner_id, activation_key, eula_accepted=False):
        '''
        Register the system with given owner_id (organization/customer) and activation_key
        '''
        #pylint: disable=W0612, W0613
        # self.unregister(silent=True)
    
        if not eula_accepted and not os.path.exists(EULA_ACCEPTED) and os.path.exists(EULA_FILE):
            # eula_text  = ''

            from unidecode import unidecode
            import codecs
            eula_text = unidecode(
                codecs
                    .open(EULA_FILE, 'r', 'utf-8')
                    .read()
            ).encode('ascii')

            # with open(EULA_FILE, 'r') as eufile_:
            #     eula_text_ = eufile_.read()
            #     udata = eula_text_.decode("utf-8")
            #     udata = udata.replace(u'“', u'"')
            #     udata = udata.replace(u'”', u'"')
            #     udata = u'\n\n' + udata
            #     eula_text = udata.encode("ascii", "ignore")

            import shlex
            rows = 40
            try:
                rows_ = int(subprocess.check_output(shlex.split('tput lines')))
                if rows_:
                    rows  = rows_
            except:
                pass
            more = More(num_lines=rows)  
            eula_text | more
            accept = ''
            while not accept:
              accept = raw_input("Accept EULA? [y/n]")
            if accept and accept[0] in ['y', 'Y']:
                eula_accepted = True

        if eula_accepted and not os.path.exists(EULA_ACCEPTED):
                with open(EULA_ACCEPTED, 'w') as lf_:
                    lf_.write('EULA ACCEPTED')
        
        eula_accepted |=  os.path.exists(EULA_ACCEPTED)

        if not eula_accepted:
            return False

        smname = self.sm_name 
        scmd = ' '.join(["%(smname)s register --activationkey ",
                         "'%(activation_key)s' --org '%(owner_id)s' --force"
                        ]) % vars()
        res = self.get_output(scmd)
        self.log.debug(res)
        if self.last_return_code != 0:
            print res    
            return False
        redhat_repo = file2string(self.repo_filepath)
        self.log.debug(redhat_repo)
        re_ = re.compile(r'(?m)baseurl\s*=\s*(?P<url>.+)\s*$')
        is_ok = self.repo_info()
        if not is_ok:
            info = '\n'.join([
                'Troubles with registration, no repository attached.',
                'If problem persists, call me with "diagnostics" options'
                ])
            self.log.error(info)
            self.log.error("owner_id=" + owner_id)
            self.log.error("activation_key=" + activation_key)
            self.log.error(self.get_consumed_products_text())
            self.log.error(self.get_current_products_text())
            self.log.error(self.get_available_products_text())
            print info
            return False
        return True
        
    @logmethod
    def unregister(self, silent=False):
        '''
        Unregistering by SM
        '''
        res = self.get_output('subscription-manager unregister')
        self.log.debug(res)
        info = ""
        if 'has been unregistered' in res:
            info = 'Successfully unregistered'
        elif 'is currently not registered' in res:
            info = 'The system already unregistered'
        else:
            info = "Troubles with deregistration. "

        self.log.info(info)

        res = self.get_output('subscription-manager clean')
        self.log.debug(res)
        
        product_cert_dir = self.cfg.get('rhsm', 'productCertDir')
        for file_ in os.listdir(product_cert_dir):
            filename = os.path.join(product_cert_dir, file_)
            os.unlink(filename)

        for file_ in [CACHE_DB, CONTRACTS_DB]:
            try:
                os.unlink(file_)
            except Exception as ex_:
                self.log.error("Failed to remove %s", file_)
                error_msg = '\n'.join([str(ex_), traceback.format_exc()])
                self.log.error(error_msg)

        if not (silent and 'already' in info):
            print info
        self.patch_sm_plugin()
        return True
        

if __name__ == '__main__':
    vzsm = VZSM()
    #vzsm.unregister()
    #vzsm.register('TheCustomer15', 'TheKey02')
    #vzsm.heal()
    #vzsm.autoattach_pools_by_contracts()
    vzsm.diagnostics()
    pass
